/*   This file is part of Xsltop.
*
*    Xsltop is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation.
*
*    Xsltop is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY.  See the
*    GNU General Public License for more details.
*
*    See <http://www.gnu.org/licenses/>.
*/
package xsltop;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.JTextPane;

import xsltop.dico.Dico;
import xsltop.dico.Labels;
import xsltop.gui.Config;
import xsltop.gui.GuiManager;
import xsltop.gui.XPathConsole;
import xsltop.gui.XmlEditor;
import xsltop.gui.XslEditor;
import xsltop.util.StreamHelper;
/**
 * Main class, gui init and creation is here. The management of the gui is
 * delegate to GuiManager. 
 * @see xsltop.gui.GuiManager
 * @author philippe voncken
 *
 */
public class Xsltop extends JFrame {		
	/** to serialization. */
	private static final long serialVersionUID = 1841396240262282199L;
	/** Name of this application. */
	public static final String NAME = "Xsltop";
	/** Version of this application. */
	public static final String VERSION = "1.0";
	/** Official site URL of this applicaiton. */
	public static final String URL = "http://xsltop.tuxfamily.org";
	/** It manage the GUI. */
	private GuiManager manager = new GuiManager(this);
	/** The default dico. */
	public static String DEFAULT_LANG = "english.dico";
	/**
	 * Default constructor.
	 * <p>Start the GUI construction.</p>
	 */
	public Xsltop() {
		build(); // the main frame
		setContentPane(buildContentPane()); // the content pane
		setJMenuBar(buildMenu()); // the menu
	}
	/**
	 * Build the main frame.
	 */
	private void build() {
		setTitle(NAME + " " + VERSION + " [" + URL + "]"); //On donne un titre à l'application
		setSize(new Integer(Config.getInstance().getWindowSizeX()),
				new Integer(Config.getInstance().getWindowSizeY())
				); //On donne une taille à notre fenêtre
		setLocationRelativeTo(null); //On centre la fenêtre sur l'écran
		setResizable(true); //On interdit la redimensionnement de la fenêtre
		// to save config on the manual resize frame
		this.addComponentListener(new ComponentListener() {  
			public void componentHidden(ComponentEvent arg0) {}
			public void componentMoved(ComponentEvent arg0) {}
			public void componentResized(ComponentEvent evt) {
				Component c = (Component)evt.getSource();
			    // Get new size
	            Dimension newSize = c.getSize();
	            Config config = Config.getInstance();
	            config.setWindowSizeX(
	            		Integer.toString((int)newSize.getWidth())
	            		);
	            config.setWindowSizeY(
	            		Integer.toString((int)newSize.getHeight())
	            		);
			}
			public void componentShown(ComponentEvent arg0) {}
		});
	}
	/**
	 * Build the content pane.
	 * @return the main panel
	 */
	private JPanel buildContentPane() {
		JPanel panelRoot = new JPanel();
		panelRoot.setLayout(new BorderLayout());
				
		final JTabbedPane tabbedpane = this.buildTabbedPane();
		//panelRoot.add(tabbedpane, BorderLayout.CENTER);
		JPanel consolepanel = new JPanel();
		JPanel console = new JPanel();
		console.setLayout(new BorderLayout());
		JTextPane consoletext = new JTextPane();
		consoletext.setEditable(false);
		console.add(consoletext, BorderLayout.CENTER);
		manager.setConsole(consoletext);		
		//consolepanel.setPreferredSize(new Dimension(400,100));
		consolepanel.setLayout(new BorderLayout());
		JScrollPane scrollConsole = new JScrollPane(console);
		console.setAutoscrolls(false);
		consolepanel.add(scrollConsole, BorderLayout.CENTER);
		//panelRoot.add(consolepanel, BorderLayout.SOUTH);
		
		final JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
				tabbedpane,
				consolepanel);
		splitPane.setDividerLocation(
				new Integer(Config.getInstance().getDividerLocation())
				);
		splitPane.setOneTouchExpandable(true);
		// to save config on the closing frame
		this.addWindowListener(
				new WindowListener(){
					public void windowActivated(WindowEvent arg0) {}
					public void windowClosed(WindowEvent arg0) {}
					public void windowClosing(WindowEvent arg0) {
						Config config = Config.getInstance();
						config.setDividerLocation(
							Integer.toString(splitPane.getDividerLocation())
						);						
						config.saveConfig();
						System.exit(0);
					}
					public void windowDeactivated(WindowEvent arg0) {}
					public void windowDeiconified(WindowEvent arg0) {}
					public void windowIconified(WindowEvent arg0) {}
					public void windowOpened(WindowEvent arg0) {}					
				});
		
		panelRoot.add(splitPane, BorderLayout.CENTER);
		return panelRoot;
	}
	/**
	 * Build the tabbedpane.
	 * @return the tabbedpane
	 */
	private JTabbedPane buildTabbedPane() {
		final JTabbedPane tabbedpane = new JTabbedPane();
		//tabbedpane.setFocusable(false);
		manager.setTabbedPane(tabbedpane);
		JPanel xmlsource = new JPanel();
		JPanel xpathconsole = new JPanel();
		JPanel xslsource = new JPanel();
		JPanel xmloutput = new JPanel();
		
		tabbedpane.addTab(
				manager.getDico().getLabel(Labels.LABEL_XML), 
				xmlsource
				);
		tabbedpane.addTab(
				manager.getDico().getLabel(Labels.LABEL_XPATH), 
				xpathconsole
				);
		tabbedpane.addTab(
				manager.getDico().getLabel(Labels.LABEL_XSL), 
				xslsource
				);
		tabbedpane.addTab(
				manager.getDico().getLabel(Labels.LABEL_XMLOUTPUT), 
				xmloutput
				);
		// XML Editor
		XmlEditor xmltextarea = new XmlEditor(getManager());
		xmltextarea.setEditable(true);
		JScrollPane xmlscroll = new JScrollPane(xmltextarea);
		xmlsource.setLayout(new BorderLayout());
		xmlsource.add(xmlscroll, BorderLayout.CENTER);
		// XPath Editor
		final JPanel filedpanel = new JPanel();
		filedpanel.setLayout(new BorderLayout());
		final JTextField xpathfield = new JTextField();
		final XPathConsole xpathtextarea = new XPathConsole(getManager());
		xpathtextarea.setEditable(false);
		JScrollPane xpathscroll = new JScrollPane(xpathtextarea);
		xpathconsole.add(xpathscroll);
		xpathconsole.setLayout(new BorderLayout());
		xpathconsole.add(xpathscroll, BorderLayout.CENTER);
		xpathconsole.add(filedpanel, BorderLayout.NORTH);
		filedpanel.add(xpathfield, BorderLayout.CENTER);
		final JButton xpathbutton = new JButton(
				manager.getDico().getLabel(Labels.LABEL_BUTTONS_XPATH)
				);
		manager.addAbstractButton(Labels.LABEL_BUTTONS_XPATH, xpathbutton);
		filedpanel.add(xpathbutton, BorderLayout.EAST);
		xpathbutton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				List<String> list = xpathtextarea.performXPath(
						xpathtextarea.interpretLine(xpathfield.getText())
						);
				xpathtextarea.setText(
						"total find: " + list.size() + "\n"
						+ xpathtextarea.listToString(list)
						);
			}			
		});
		xpathfield.addKeyListener(new KeyListener(){
			public void keyPressed(KeyEvent e) {
				if (10 == e.getKeyCode()) { // si Enter
					List<String> list = xpathtextarea.performXPath(
							xpathtextarea.interpretLine(xpathfield.getText())
							);
					xpathtextarea.setText(
							"total find: " + list.size() + "\n"
							+ xpathtextarea.listToString(list)
							);
				}
			}
			public void keyReleased(KeyEvent e) {
			}
			public void keyTyped(KeyEvent e) {
			}			
		});
		// XSL PANE
		XslEditor xsltextarea = new XslEditor(getManager());
		xmltextarea.setEditable(true);
		JScrollPane xslscroll = new JScrollPane(xsltextarea);
		xslsource.setLayout(new BorderLayout());
		xslsource.add(xslscroll, BorderLayout.CENTER);
		// XML OUTPUT
		JTextPane xmloutputtextarea = new JTextPane();
		manager.setXmlOutput(xmloutputtextarea);
		xmloutputtextarea.setEditable(true);
		JScrollPane xmloutputscroll = new JScrollPane(xmloutputtextarea);
		xmloutput.setLayout(new BorderLayout());
		xmloutput.add(xmloutputscroll, BorderLayout.CENTER);
		return tabbedpane;
	}
	/**
	 * Build the menu.
	 * @return the menu bar
	 */
	private JMenuBar buildMenu() {
		
		JMenuBar menubar = new JMenuBar();
		// MENU FILE **************************
		final JMenu xmlfile = buildMenuXmlFile();
		menubar.add(xmlfile);
		final JMenu xslfile = buildMenuXslFile();
		menubar.add(xslfile);
		// MENU EDIT ***********************
		JMenu edit = new JMenu();
		menubar.add(edit);
		manager.addAbstractButton(Labels.MENU_EDIT, edit);
		
		JMenu trace = buildMenuEditTrace();
		JMenu languages = buildMenuEditLanguages();
		JMenu xmloptions = buildMenuEditXmlOptions();
		edit.add(languages);
		edit.add(trace);
		edit.add(xmloptions);
		return menubar;
	}
	/**
	 * Build the menu edit xml options item to add to the menu.
	 * @return a JMenu
	 */
	private JMenu buildMenuEditXmlOptions() {
		JMenu xmloptions = new JMenu(
				manager.getDico().getLabel(Labels.MENU_XMLOPTIONS)
				);
		manager.addAbstractButton(Labels.MENU_XMLOPTIONS, xmloptions);
		final JCheckBox namespace = new JCheckBox(
				manager.getDico().getLabel(Labels.MENU_NAMESPACEAWARE)
				);
		namespace.setSelected(new Boolean(
				Config.getInstance().getNamespaceaware())
		);
		manager.addAbstractButton(Labels.MENU_NAMESPACEAWARE, namespace);
		xmloptions.add(namespace);
		namespace.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent evt) {
				manager.setNamespaceaware(namespace.isSelected());
				if (namespace.isSelected()) {
					manager.getLogger().info(manager.getDico().getLabel(
							Labels.MSG_NAMESPACEAWARE)
							);
				} else {
					manager.getLogger().info(manager.getDico().getLabel(
							Labels.MSG_NAMESPACENOTAWARE)
							);
				}
			}
		});
		return xmloptions;
	}
	/**
	 * Build the menu edit trace item.
	 * @return a JMenu
	 */
	private JMenu buildMenuEditTrace() {
		String level = Config.getInstance().getLogLevel();
		Config.getInstance().setLogLevel(level);
		JMenu trace = new JMenu(
				manager.getDico().getLabel(Labels.MENU_TRACE)
				);
		manager.addAbstractButton(Labels.MENU_TRACE, trace);
		ButtonGroup group = new ButtonGroup();
		
		final JRadioButtonMenuItem debug = new JRadioButtonMenuItem(
				manager.getDico().getLabel(Labels.MENU_TRACE_DEBUG)
				);
		debug.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.getLogger().setLevelToDebug();
				Config.getInstance().setLogLevel("debug");
			}
		});
		final JRadioButtonMenuItem info = new JRadioButtonMenuItem(
				manager.getDico().getLabel(Labels.MENU_TRACE_INFO)
				);
		info.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.getLogger().setLevelToInfo();
				Config.getInstance().setLogLevel("info");
			}
		});
		final JRadioButtonMenuItem error = new JRadioButtonMenuItem(
				manager.getDico().getLabel(Labels.MENU_TRACE_ERROR)
				);
		error.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				manager.getLogger().setLevelToError();
				Config.getInstance().setLogLevel("error");
			}
		});
		group.add(debug);
		trace.add(debug);
		if ("debug".equals(level)) {
			debug.setSelected(true);
			manager.getLogger().setLevelToDebug();
		}
		group.add(info);
		trace.add(info);
		if ("info".equals(level)) {
			info.setSelected(true);
			manager.getLogger().setLevelToInfo();
		}
		group.add(error);
		trace.add(error);
		if ("error".equals(level)) {
			error.setSelected(true);
			manager.getLogger().setLevelToError();
		}
		return trace;
	}
	/**
	 * Build the menu xml file item.
	 * @return a JMenu
	 */
	private JMenu buildMenuXmlFile() {
		final JMenu file = new JMenu();
		manager.addAbstractButton(Labels.MENU_XML_FILE, file);		
		// --------------------- OpenFile
		JMenuItem openfile = new JMenuItem();		
		openfile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getManager().openXmlFile();				
			}			
		});
		manager.addAbstractButton(Labels.MENU_XML_FILE_OPEN, openfile);
		file.add(openfile);
		// --------------------- SaveFile
		JMenuItem savefile = new JMenuItem();		
		savefile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getManager().saveXML();				
			}			
		});
		manager.addAbstractButton(Labels.MENU_XML_FILE_SAVE, savefile);
		file.add(savefile);
		// --------------------- SaveFileAs
		JMenuItem savefileas = new JMenuItem();		
		savefileas.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getManager().saveXMLas();				
			}			
		});
		manager.addAbstractButton(Labels.MENU_XML_FILE_SAVEAS, savefileas);
		file.add(savefileas);
		return file;
	}
	/**
	 * Build the menu xsl file item.
	 * @return a JMenu
	 */
	private JMenu buildMenuXslFile() {
		final JMenu file = new JMenu();
		manager.addAbstractButton(Labels.MENU_XSL_FILE, file);		
		// --------------------- OpenFile
		JMenuItem openfile = new JMenuItem();		
		openfile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getManager().openXslFile();				
			}			
		});
		manager.addAbstractButton(Labels.MENU_XSL_FILE_OPEN, openfile);
		file.add(openfile);
		// --------------------- SaveFile
		JMenuItem savefile = new JMenuItem();		
		savefile.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getManager().saveXSL();				
			}			
		});
		manager.addAbstractButton(Labels.MENU_XSL_FILE_SAVE, savefile);
		file.add(savefile);
		// --------------------- SaveFileAs
		JMenuItem savefileas = new JMenuItem();		
		savefileas.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				getManager().saveXSLas();				
			}			
		});
		manager.addAbstractButton(Labels.MENU_XSL_FILE_SAVEAS, savefileas);
		file.add(savefileas);
		return file;
	}
	/**
	 * Build the menu edit language item.
	 * @return a JMenu
	 */
	private JMenu buildMenuEditLanguages() {
		JMenu languages = new JMenu();
		manager.addAbstractButton(Labels.MENU_LANGUAGES, languages);
		// ---------------------------------- LANG
		List<String> list = Config.getInstance().getLang();
		list = StreamHelper.selectDicoFile(list);	
		
		ButtonGroup group = new ButtonGroup();
		for (final String dico:list) {
			final JRadioButtonMenuItem language = new JRadioButtonMenuItem(
				Dico.getInstance(dico).getLabel(Labels.DICO_LANG_LABEL)
				);
			language.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					manager.setDico(dico);
					Config.getInstance().setLangDefault(dico);
					Config.getInstance().saveConfig();
				}
			});
			if (dico.equals(Config.getInstance().getLangDefault())) { 
				// le dictionnaire par defaut
				language.setSelected(true);
				manager.setDico(dico);
			}
			group.add(language);
			languages.add(language);
		}
		return languages;
	}
	/**
	 * Main method.
	 * @param args no params
	 */
	public static void main(String[] args) {
		Xsltop cookxml = new Xsltop();
		cookxml.setVisible(true);
	}
	/**
	 * @return the GUI manager
	 */
	private GuiManager getManager() {
		return manager;
	}	
}
